# coding: utf-8

"""
    Voucherify API

    Voucherify promotion engine REST API. Please see https://docs.voucherify.io/docs for more details.

    The version of the OpenAPI document: v2018-08-01
    Contact: support@voucherify.io
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictInt, StrictStr, field_validator
from typing import Any, ClassVar, Dict, List, Optional
from typing_extensions import Annotated
from typing import Optional, Set
from typing_extensions import Self

class AsyncActionBase(BaseModel):
    """
    This is an object representing an asynchronous action.
    """ # noqa: E501
    id: Optional[StrictStr] = Field(default=None, description="Async action unique ID.")
    type: Optional[StrictStr] = Field(default=None, description="Type of async action.")
    status: Optional[StrictStr] = Field(default=None, description="Status of the async action. Informs you whether the async action has already been completed.")
    operation_status: Optional[StrictStr] = Field(default=None, description="Status of async action processing. Informs about the async action status, whether it failed, succeeded, or the status is unknown.")
    created_at: Optional[datetime] = Field(default=None, description="Timestamp representing the date and time when the async action was scheduled in ISO 8601 format.")
    updated_at: Optional[datetime] = Field(default=None, description="Timestamp representing the date and time when the async action was updated. The value is shown in the ISO 8601 format.")
    request_id: Optional[StrictStr] = Field(default=None, description="Unique request ID.")
    processing_time: Optional[StrictInt] = Field(default=None, description="The length of time it took to process the request in milliseconds.")
    progress: Optional[Annotated[int, Field(le=100, strict=True, ge=0)]] = Field(default=None, description="% progress to completion of the asynchronous action.")
    object: Optional[StrictStr] = Field(default='async_action', description="The type of the object represented by JSON. This object stores information about the `async_action`.")
    __properties: ClassVar[List[str]] = ["id", "type", "status", "operation_status", "created_at", "updated_at", "request_id", "processing_time", "progress", "object"]

    @field_validator('status')
    def status_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in set(['DONE', 'ENQUEUED', 'FAILED', 'IN_PROGRESS']):
            raise ValueError("must be one of enum values ('DONE', 'ENQUEUED', 'FAILED', 'IN_PROGRESS')")
        return value

    @field_validator('object')
    def object_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in set(['async_action']):
            raise ValueError("must be one of enum values ('async_action')")
        return value

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of AsyncActionBase from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # set to None if id (nullable) is None
        # and model_fields_set contains the field
        if self.id is None and "id" in self.model_fields_set:
            _dict['id'] = None

        # set to None if type (nullable) is None
        # and model_fields_set contains the field
        if self.type is None and "type" in self.model_fields_set:
            _dict['type'] = None

        # set to None if status (nullable) is None
        # and model_fields_set contains the field
        if self.status is None and "status" in self.model_fields_set:
            _dict['status'] = None

        # set to None if operation_status (nullable) is None
        # and model_fields_set contains the field
        if self.operation_status is None and "operation_status" in self.model_fields_set:
            _dict['operation_status'] = None

        # set to None if created_at (nullable) is None
        # and model_fields_set contains the field
        if self.created_at is None and "created_at" in self.model_fields_set:
            _dict['created_at'] = None

        # set to None if updated_at (nullable) is None
        # and model_fields_set contains the field
        if self.updated_at is None and "updated_at" in self.model_fields_set:
            _dict['updated_at'] = None

        # set to None if request_id (nullable) is None
        # and model_fields_set contains the field
        if self.request_id is None and "request_id" in self.model_fields_set:
            _dict['request_id'] = None

        # set to None if processing_time (nullable) is None
        # and model_fields_set contains the field
        if self.processing_time is None and "processing_time" in self.model_fields_set:
            _dict['processing_time'] = None

        # set to None if progress (nullable) is None
        # and model_fields_set contains the field
        if self.progress is None and "progress" in self.model_fields_set:
            _dict['progress'] = None

        # set to None if object (nullable) is None
        # and model_fields_set contains the field
        if self.object is None and "object" in self.model_fields_set:
            _dict['object'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of AsyncActionBase from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "id": obj.get("id"),
            "type": obj.get("type"),
            "status": obj.get("status"),
            "operation_status": obj.get("operation_status"),
            "created_at": obj.get("created_at"),
            "updated_at": obj.get("updated_at"),
            "request_id": obj.get("request_id"),
            "processing_time": obj.get("processing_time"),
            "progress": obj.get("progress"),
            "object": obj.get("object") if obj.get("object") is not None else 'async_action'
        })
        return _obj


